import pygame
from pygame.locals import *
import serial
import threading



### Settings
VERSION = "SwH 2014b - UGV Controller"
BG_COLOR = (0x20, 0x20, 0x20)	#(0x33, 0xcc, 0x99)
SCREEN_SIZE = (640, 480)
SCREEN_HALF = (SCREEN_SIZE[0]/2, SCREEN_SIZE[1]/2)

SERIAL_ENABLED = True	# You can disable it for debugging "offline"
SERIAL_PORT = "COM11"	# COM-port of Arduino OR Bluetooth Connection to HC-05
SERIAL_BAUDRATE = 38400


STATE_F = 0
STATE_B = 1
STATE_L = 2
STATE_R = 3

STATES = 4
states = [0,0,0,0]
STATE_POS = [
	[0,-1],
	[0,1],
	[-1,0],
	[1,0]
]

### Simple text output (console and GUI)
lastPut = ''
def put(txt):
	print(VERSION + ':	' + str(txt))
	global lastPut
	lastPut = txt

### Serial communication
ser = None

if SERIAL_ENABLED:
	#try:
	put('Opening SERIAL_PORT "' + SERIAL_PORT + '"...')
	ser = serial.Serial(port=SERIAL_PORT, baudrate=SERIAL_BAUDRATE, timeout=1)
	#except e:
	#put('Error opening serial device ' + SERIAL_PORT + ': ' + str(e))

if ser == None:
	put("Port is not open!")

def serialDoCommand(t):
	put('>> "' + t + '"...')
	if (ser):
		ser.write(t)

def serialHandleData(data):
	put('<< "' + data + '"')

def serialReader(ser):
	while ser.isOpen():
		data = ser.readline().decode()
		if (data): serialHandleData(data)

if (ser):
	thread = threading.Thread(target=serialReader, args=(ser,))
	thread.start()


### Init PyGame
pygame.init()
screen = pygame.display.set_mode(SCREEN_SIZE, 0, 32)
#screen.fill((0x00, 0x00, 0x00))
clock = pygame.time.Clock()
font = pygame.font.SysFont("Arial", 12, True)
pygame.display.set_caption(VERSION)
pygame.mouse.set_visible(False)


running = True
put("Running...")

while running:
	#pygame.time.delay(100/2)
	#
	tick = clock.tick()
	
	for event in pygame.event.get():
		if event.type == QUIT:
			exit()
		
		if event.type == MOUSEBUTTONDOWN:
			exit()
			pass
		
		if event.type == MOUSEMOTION:
			#pygame.draw.line(screen, (0x33, 0xcc, 0x99), (oldMousePos[0], oldMousePos[1]), (event.pos[0], event.pos[1]), 2)
			#oldMousePos = event.pos
			#trace("x=" + str(event.pos[0]) + ", y=" + str(event.pos[1]), 0, False)
			pass
			
		
		if event.type == KEYDOWN:
			
			# Check for key down
			if (event.key == K_UP) or (event.key == ord('w')):
				serialDoCommand('F')
				states[STATE_F] = 1
			
			elif (event.key == K_DOWN) or (event.key == ord('s')):
				serialDoCommand('B')
				states[STATE_B] = 1
			
			elif (event.key == K_LEFT) or (event.key == ord('a')):
				serialDoCommand('L')
				states[STATE_L] = 1
			
			elif (event.key == K_RIGHT) or (event.key == ord('d')):
				serialDoCommand('R')
				states[STATE_R] = 1
				
			elif (event.key == K_BACKSPACE):
				serialDoCommand('S')
				states[STATE_F] = 0
				states[STATE_B] = 0
				states[STATE_L] = 0
				states[STATE_R] = 0
			elif (event.key == K_1):
				serialDoCommand('1')
			
			elif (event.key == K_2):
				serialDoCommand('2')
			
			elif (event.key == K_3):
				serialDoCommand('3')
				
			elif (event.key == K_4):
				serialDoCommand('4')
			
			elif (event.key == K_5):
				serialDoCommand('5')
			
			elif (event.key == K_6):
				serialDoCommand('6')
			
			elif (event.key == K_7):
				serialDoCommand('7')
			
			elif (event.key == K_8):
				serialDoCommand('8')
			
			elif (event.key == K_9):
				serialDoCommand('9')
			
			elif (event.key == K_0):
				serialDoCommand('0')
			
			elif (event.key == K_ESCAPE):	#27):	#esc
				running = False
			
			else:
				put('Unknown key: ' + str(event.key))
			
		# Check for key release
		if event.type == KEYUP:
			if (event.key == K_UP) or (event.key == ord('w')):
				serialDoCommand('f')
				states[STATE_F] = 0
				
			elif (event.key == K_DOWN) or (event.key == ord('s')):
				serialDoCommand('b')
				states[STATE_B] = 0
				
			elif (event.key == K_LEFT) or (event.key == ord('a')):
				serialDoCommand('l')
				states[STATE_L] = 0
				
			elif (event.key == K_RIGHT) or (event.key == ord('d')):
				serialDoCommand('r')
				states[STATE_R] = 0
				
		# end of event loop
	
	"""
	# Check for pressed keys
	keys = pygame.key.get_pressed()
	if keys[pygame.K_UP]: put('up')
	if keys[pygame.K_DOWN]: put('down')
	if keys[pygame.K_LEFT]: put('left')
	if keys[pygame.K_RIGHT]: put('right')
	"""
	
	
	screen.fill(BG_COLOR)
	
	#col = (0x00, 0x00, 0x00)
	#pygame.draw.circle(screen, col, (int((i+0.5) * 160), SCREEN_SIZE[1]/2), 80, 0)
	
	# Visualize the main states
	wh = SCREEN_SIZE[0] /3
	hh = SCREEN_SIZE[1] /3
	r = int(SCREEN_SIZE[1]/12)
	ap = [0,0]
	for i in xrange(STATES):
		p = STATE_POS[i]
		s = states[i]
		x = SCREEN_HALF[0] + p[0]*wh
		y = SCREEN_HALF[1] + p[1]*hh
		col = (0xff, 0xff, 0xff)
		if (s == 0):
			pygame.draw.circle(screen, col, (int(x), int(y)), r, 2)
		else:
			pygame.draw.circle(screen, col, (int(x), int(y)), r, 0)
			ap[0] += p[0]
			ap[1] += p[1]
			
	# Show direction
	x = SCREEN_HALF[0] + ap[0]*wh
	y = SCREEN_HALF[1] + ap[1]*hh
	pygame.draw.line(screen, (0x33, 0xcc, 0x99), (SCREEN_HALF[0], SCREEN_HALF[1]), (x, y), 4)
	
	# Render text output
	putImage = font.render(lastPut, True, (255,255,255))
	rect = putImage.get_rect()
	rect[0] = 10
	rect[1] = 10
	screen.blit(putImage, rect)
	
	pygame.display.update()
	
put("Finished.")
#sys.exit()
if (ser):
	ser.close()
	ser = None
